*:*****************************************************
*:
*:   Program: SNAKEIT.PRG     :: a procedure file
*:   Purpose: Multiple records per line in reports
*:   Author:  Lisa C. Slater
*:   Copyright (c) 1990, SoftSpoken Santa Rosa CA
*:            
*:   Fncts: DO_SKIP() :: chain items across page
*:        : SNAKE2()  :: snake all items down by page
*:        : SNAKE3()  :: snake items by group by page
*:        : D_LINES() :: figure detail lines per page
*:
*:
*:*****************************************************


*:*****************************************************
*:
*:  DO_SKIP() :: chain items across
*:
*:*****************************************************

** Before calling the report:
** Name your group expression:
*  whichgroup = "test->group" 
** If you're not grouping, just assign a dummy variable
** that will never change value during the report run:
*  mdummy = "X"
*  whichgroup = "mdummy"

FUNCTION do_skip
mreturn = .T.
oldvalue = &whichgroup
SKIP 
* if we've changed group, need to let Report Writer
* do its own break
* need a check for EOF() just in case we're not grouping
* so the first condition will never be true
IF &whichgroup <> oldvalue .OR. EOF()
   SKIP -1
   mreturn = .F.
ENDIF
RETURN mreturn


*:*****************************************************
*:
*: SNAKE2() :: snake all items down
*:
*:*****************************************************

** Before calling report:
*  no_rows = d_lines("test2.frx")
** see D_LINES() for additional comments
*  rowcount = 1  && must be 1
*  whatprint = "test->group+test->thing" && print expr

FUNCTION snake2
PARAMETERS no_col, no_cols
mreturn = ""
IF ! EOF()
   whichrec = RECNO()
   SKIP no_rows*(no_col-1)
   IF ! EOF()
      mreturn = &whatprint
   ENDIF
   IF no_cols = no_col
      rowcount = rowcount + 1
   ENDIF      
   IF rowcount = no_rows + 1
      rowcount = 1
   ENDIF
   IF ! (rowcount = 1 .AND. no_cols = no_col)
      GO whichrec
   ENDIF       
ENDIF
RETURN mreturn


*:*****************************************************
*:
*:  SNAKE3() :: snake items by group
*:
*:*****************************************************


** Report form *must* start a new page on the group expr
** Before calling report:
*  no_rows = d_lines("reportname.frx")
** see D_LINES() for additional comments
*  rowcount = 1  && must be 1
*  whatprint = "test->thing+test->group" && print expr
*  whichgroup = "test->group"            && group expr

FUNCTION snake3
PARAMETERS no_col, no_cols
mreturn = ""
IF ! EOF()
   whichrec = RECNO()
   thisgroup = &whichgroup
   SKIP no_rows*(no_col-1)
   IF &whichgroup == thisgroup .AND. ! EOF()
      mreturn = &whatprint
      IF no_cols = no_col && end of the line
         rowcount = rowcount + 1 
         IF rowcount = no_rows + 1 
            rowcount = 1
         ENDIF
      ENDIF
    ELSE
       IF no_cols = no_col && end of the line
          rowcount = rowcount + 1
          IF rowcount = no_rows + 1
             rowcount = 1
             IF &whichgroup <> thisgroup
                * need to let Report Writer do skip
                * as a break on the group
                DO WHILE &whichgroup <> thisgroup
                   SKIP -1
                ENDDO            
             ENDIF                
          ENDIF                     
       ENDIF         
    ENDIF
    IF ! (rowcount = 1 .AND. no_cols = no_col)
       GO whichrec
       * otherwise let Report Writer do skip
    ENDIF       
ENDIF
RETURN mreturn


*:*****************************************************
*:
*:  D_LINES() :: figure detail lines per page
*:
*:*****************************************************

* Doesn't work with blank header or footer lines!
* If you're stretching vertically (more than one line
* per expression), or want more than one block on the pg,
* no_rows will be d_lines("report.frx")
* divided by no_lines_ per_expression, must come out even.  

FUNCTION d_lines
PARAMETERS mreport
rhandle = FOPEN(mreport)  && read only
mreturn = 0
lcheck = " "
firsttime = .T.
IF rhandle # -1
  DO WHILE ! FEOF(rhandle)
    rline = FGETS(rhandle)
    objtype = VAL(SUBSTR(rline,1,2))
    IF objtype = 18  && band description
      LOOP
    ENDIF
    * get positions of significant tabs on the line
    t4 = AT(CHR(9),rline,4)      
    t5 = AT(CHR(9),rline,5)
    t6 = AT(CHR(9),rline,6)
    t7 = AT(CHR(9),rline,7)
    IF objtype = 1
      * add pagelength, subtr top & bottom margins
      mreturn = mreturn + VAL(SUBSTR(rline,t6+1,t7-t6));  
       - (VAL(SUBSTR(rline,t5+1,t6-t5));
       + VAL(SUBSTR(rline,t4+1,t5-t4)))   
    ELSE
      * check to see if we've already dealt with this line
      * if not, subtract the height of this object from
      * available detail lines
      IF !(ALLTRIM(SUBSTR(rline,t4+1,t5-t4)) $ lcheck);
        .AND. ! firsttime
        lcheck = lcheck + ALLTRIM(SUBSTR(rline,t4+1,t5-t4))
        rspace = VAL(SUBSTR(rline,t6+1,t7-t6))      
        mreturn = mreturn - IIF(rspace = 0,1,rspace)
        firsttime = .F.                     
      ELSE            
        lcheck = lcheck + ALLTRIM(SUBSTR(rline,t4+1,t5-t4))         
        firsttime = .F.         
      ENDIF            
    ENDIF         
  ENDDO      
ENDIF
= FCLOSE(rhandle)
RETURN mreturn   

*: EOF:  SNAKEIT.PRG
