  1   Proc Wait_Record_Demo()       ;demonstrates WAIT RECORD processing
  2   
  3   Private 
  4      after_rec,        ;the record after  the WAIT, from CopyToArray
  5      before_rec,       ;the record before the WAIT, from CopyToArray
  6      is_arriving,      ;whether we're arriving at a new record
  7      is_modified,      ;whether the record has been modified
  8      is_readonly,      ;whether the record is in a read-only state
  9      wait_key,         ;the "Until" key pressed in response to the WAIT
 10      wait_msg          ;the error message to display on the WAIT command
 11   
 12   ;  Intialization
 13   
 14   Reset                   ;clear the workspace
 15   CoEdit "Customer"       ;view CUSTOMER table; enter co-edit mode
 16   is_arriving = True      ;we begin by arriving at a new record
 17   wait_msg = ""           ;no error message yet
 18   
 19   ;  This is the WAIT loop
 20   
 21   While True     ;the WAIT command will be executed repeatedly.
 22   
 23   ;  Step 1.  Arrival Processing.
 24   
 25      If is_arriving then           ;we just arrived at a new record
 26         is_arriving = False        ;arrival is being handled now
 27         If not recordstatus("New") and      ;it's an existing record
 28            not recordstatus("Locked") then  ;but not yet locked
 29            LockRecord                       ;attempt to lock the record
 30            If not retval then               ;the lock failed
 31               wait_msg = errormessage()     ;display the error message
 32            Endif
 33         Endif
 34         If recordstatus("New") or        ;it's a new record, or
 35            recordstatus("Locked") then   ;it's a locked record
 36            is_readonly = False           ;let the user modify it
 37            CopyToArray before_rec        ;remember record, pre-change
 38            Record_Arrival()              ;perform the arrival proc
 39            If not retval then            ;error in arrival proc
 40               is_readonly = True         ;don't let user change record
 41               is_arriving = True         ;repeat arrival proc next loop
 42            Endif
 43         Else                             ;an existing record, not lockd
 44            is_readonly = True            ;don't let the user modify it
 45         Endif
 46         is_modified = recordstatus("Modified")    ;is rcd modified yet?
 47      Endif
 48      If is_readonly then           ;if user may not change the record
 49         ImageRights ReadOnly       ;prohibit changes
 50      Endif
 51   
 52   ;  Step 2.  The WAIT RECORD Command.
 53   
 54      Wait Record
 55         Prompt "Editing the "+table()+" table",
 56                "[F2]=Done Editing"
 57         Message wait_msg
 58         Until "Do_it!",                     ;termination key
 59               "Up","Down","PgUp","PgDn",    ;movement keys
 60               "Home","End","Ins",           ;movement keys
 61               "Help","FieldView","F35",     ;non-movement keys
 62               "Dos","LockKey"               ;illegal keys
 63   
 64      wait_key = retval    ;remember which UNTIL key the user pressed
 65      wait_msg = ""        ;clear the previous error message
 66      ImageRights          ;restore full rights, if they were ReadOnly
 67      If recordstatus("Modified") then    ;did the user modify it?
 68         is_modified = True               ;yes; remember it's modified
 69      Endif
 70   
 71   ;  Step 3.  Process Non-Movement Function Keys.
 72   
 73      Non_Move_Keys()               ;process the non-movement keys
 74      If retval then Loop Endif     ;no further work needed
 75   
 76   ;  Step 4.  Departure Processing.
 77   
 78      If recordstatus("KeyViol") then        ;we still have dup key
 79         wait_msg = "Fix the duplicate key!" ;the error message
 80         Loop                                ;do not proceed
 81      Endif
 82      CopyToArray after_rec               ;remember the new record
 83      If not is_readonly then             ;record could be modified
 84         Record_Departure()               ;perform the departure proc
 85         If not retval then Loop Endif    ;trouble with this record
 86      Endif
 87      is_arriving = True  ;next loop will be a new record arrival
 88   
 89   ;  Step 5.  Movement Key Processing.
 90   
 91      Move_Keys()                ;process the keys
 92   
 93   ;  Step 6.  Possible Termination.
 94   
 95      If sysmode()="Main" then QuitLoop Endif
 96   
 97   ;  Step 7.  Possible Key-Violation.
 98   
 99      If recordstatus("KeyViol") then     ;it's a duplicate key
100         wait_msg = "Another record with this key already exists"
101      Endif
102   EndWhile
103   
104   EndProc
105   
106   ;  ------------------------------------------------------------------
107   
108   Proc Non_Move_Keys()  ;process keys which don't move from the record
109   
110   Switch
111      Case wait_key="Help":         ;F1 -- Lookup-Help
112         If is_readonly then Beep   ;we're preventing record changes
113         Else Lookup_Echo() Endif   ;refer to Feb. '91 DBA
114      Case wait_key="FieldView" or wait_key="F35":    ;Ctrl-F, Alt-F5
115         Wait_FieldView()           ;refer to Jan. '91 DBA, p. 48
116      Case wait_key="Dos" or wait_key="LockKey":   Beep  ;illegal keys
117      Otherwise:                    ;it's a record-movement key
118         Return False               ;signal for record movement
119   EndSwitch
120   
121   Return True    ;return, and signal that the key was processed
122   
123   EndProc
124   
125   ;  ------------------------------------------------------------------
126   
127   Proc Move_Keys()     ;process the keys which cause record movement
128   
129   Switch
130      Case wait_key="Do_it!":    ;F2- terminate the process
131         Do_it!
132      Otherwise:                 ;"PgUp", "PgDn", "Ins", etc.
133         KeyPress wait_key
134   EndSwitch
135   
136   EndProc
137   
138   ;  ------------------------------------------------------------------
139   
140   Proc Record_Arrival()   ;processing when you arrive at a new record
141   
142   Message "Insert your record-arrival procedure here..."
143   Sleep 500
144   
145   Return True    ;return TRUE for success, FALSE for failure
146   
147   EndProc
148   
149   ;  ------------------------------------------------------------------
150   
151   Proc Record_Departure()  ;processing before departing from a record
152   
153   Message "Insert your record-departure procedure here..."
154   Sleep 500
155   
156   Return True    ;return TRUE for success, FALSE for failure
157   
158   EndProc
159   
160   ;  ------------------------------------------------------------------
161   
162   Wait_Record_Demo()              ;execute the main procedure
163   
164   ;  ------------------------------------------------------------------
